% Set up parameters
Nx = 256;
Ny = 256;
accel = 6;
plot_scale = 64;
psf = zeros(Nx,Ny);
psf(5,8) = 1;
wavelet_op = Wavelet('Daubechies',4,6);

% Plot point spread function (PSF)
figure; s1 = surf(psf(1:plot_scale,1:plot_scale));
s1.EdgeColor = 'r';
s1.FaceColor = 'interp';
title('Sparse signal in Wavelet domain','FontSize',20)

% Plot PSF in image domain
psf_img = wavelet_op'*psf;
figure; s2 = surf(psf_img(1:plot_scale,1:plot_scale));
s2.EdgeColor = 'r';
s2.FaceColor = 'interp';
title('Corresponding signal in image domain','FontSize',20)

% From image to k-space
psf_kspace = ifftshift(fft2(fftshift(psf_img)));

% Generate equidistance undersampling mask
mask_equi = zeros(Nx,Ny);
mask_equi(1:accel:end,:) = 1;

% Generate random undersampling mask
mask_rand = zeros(Nx,Ny);
rng(0)
rand_idx = randperm(Nx*Ny);
rand_idx = rand_idx(1:floor(Nx*Ny/accel));
mask_rand(rand_idx) = 1;

figure;imshow(cat(2, mask_equi, mask_rand),[])
title('Equidistance vs. Random undersampling', 'FontSize',20);

% Apply undersampling mask
psf_kspace_equi = psf_kspace.*mask_equi;
psf_kspace_rand = psf_kspace.*mask_rand;

% From undersampled k-space to image
psf_img_equi = fftshift(ifft2(ifftshift(psf_kspace_equi)));
psf_img_rand = fftshift(ifft2(ifftshift(psf_kspace_rand)));

% Plot results from different undersampling pattern
figure; s3 = surf(abs(psf_img_equi(1:plot_scale,1:plot_scale)));
s3.EdgeColor = 'r';
s3.FaceColor = 'interp';
title('Images from equidistance undersampling','FontSize',20)

figure; s4 = surf(abs(psf_img_rand(1:plot_scale,1:plot_scale)));
s4.EdgeColor = 'r';
s4.FaceColor = 'interp';
title('Images from random undersampling','FontSize',20)


% From image to Wavelet domain
psf_wl_equi = wavelet_op*psf_img_equi;
psf_wl_rand = wavelet_op*psf_img_rand;

% Plot results from different undersampling pattern
figure; s5 = surf(abs(psf_wl_equi(1:plot_scale,1:plot_scale)));
s5.EdgeColor = 'r';
s5.FaceColor = 'interp';
title('Wavalet coefficients from equidistance undersampling','FontSize',20)

figure; s6 = surf(abs(psf_wl_rand(1:plot_scale,1:plot_scale)));
s6.EdgeColor = 'r';
s6.FaceColor = 'interp';
title('Wavalet coefficients from random undersampling','FontSize',20)

